// Copyright (c) <2012> <Leif Asbrink>
//
// Permission is hereby granted, free of charge, to any person 
// obtaining a copy of this software and associated documentation 
// files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of 
// the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be 
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
// OR OTHER DEALINGS IN THE SOFTWARE.


#include <stdio.h>
#include <malloc.h>
#include <wchar.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "globdef.h"
#if(OSNUM == OSNUM_WINDOWS)
  #include <windows.h>
#else
  #include <unistd.h>
  #include <sys/types.h>	
  #include <sys/stat.h>
  #include <fcntl.h>
  #include <errno.h>
#endif

#include "uidef.h"
#include "sdrdef.h"
#include "vernr.h"
#include "screendef.h"
#include "hidapi.h"



#define RF_GAIN_ZERO    -10
#define MAX_STR 255
	int GainIndex[7]={0,4,2,6,1,5,3};
	double GainTable[7]={1.00, 1.14, 1.33, 1.60, 2.00, 2.67, 4};
        int FE_Gain = 0;
	volatile int hid_status=0;
        wchar_t wstr[MAX_STR+1];
        int HID_COMMAND_SIZE = 7;
// set but not used int global_device_PID = HID_DEVICE_SDR_NET_PID;
        unsigned char hid_tcp_packet[32]={0};
        volatile int hid_tcp_network_flag = 0;
        volatile int hid_command_send_state = 0;
        char Gpath_name[MAX_STR+1]={0};
        wchar_t USB_Serial_number[MAX_STR+1]=L"\0";
        wchar_t *SerialNumber = (wchar_t *)NULL;



#define DIVERSITY_MODE 1
#define DUAL_CHANNEL_MODE 2
#define DUAL_CHANNEL_MODE_OFF 0
#define DIVERSITY_INTERNAL_ADD_MODE 3
#define MSD_CLUSTER_SIZE 512
#define HID_COMMAND_MAX_SIZE 7
#define AFEDRI8201_MAIN_CLOCK 66.00e6
#define TRANSVERTER_MODE_LIMIT  1200000000
#define NCO_MULTIPLIER 4294967296.00
#define HID_MEMORY_READ_WRITE_REPORT 1
#define HID_FREQUENCY_REPORT 3
#define HID_GENERIC_REPORT 2
#define HID_GENERIC_GAIN_COMMAND 2
#define HID_GENERIC_INIT_FE_COMMAND 0xE1
#define HID_GENERIC_GET_FREQ_COMMAND 1
#define HID_GENERIC_GET_INIT_STATUS_COMMAND 4
#define HID_GENERIC_DAC_COMMAND 8
#define HID_GENERIC_VER_COMMAND 9
#define HID_GENERIC_CONSOLE_IO_COMMAND 10
#define HID_GENERIC_HID_CONSOLE_ON 11
#define HID_READ_EEPROM_COMMAND 0x55
#define HID_WRITE_EEPROM_COMMAND 0x56
#define HID_GENERIC_SAVE_DEFAULT_PARAMS_COMMAND 12
#define HID_GENERIC_GET_SR_COMMAND 14
#define HID_GENERIC_GET_DIVERSITY_MODE 15
#define HID_GENERIC_GET_SDR_IP_ADDRESS 16
#define HID_GENERIC_GET_SDR_IP_MASK 17
#define HID_GENERIC_GET_GATEWAY_IP_ADDRESS 18
#define HID_GENERIC_GET_DST_IP_ADDRESS 19
#define HID_GENERIC_GET_DST_PORT 20
#define HID_GENERIC_SAVE_SDR_IP_ADDRESS 21
#define HID_GENERIC_SAVE_SDR_IP_MASK 22
#define HID_GENERIC_SAVE_GATEWAY_IP_ADDRESS 23
#define HID_GENERIC_SAVE_DST_IP_ADDRESS 24
#define HID_GENERIC_SAVE_DST_PORT 25
#define HID_GENERIC_START_UDP_STREAM_COMMAND 26
#define HID_GENERIC_OVERLAOD_STATE 27
#define HID_GENERIC_ACK 28
#define HID_GENERIC_GET_SERIAL_NUMBER_COMMAND 29
#define HID_GENERIC_SET_SAMPLE_RATE_COMMAND 30
#define HID_GENERIC_SET_VCO_COMMAND 31
#define HID_GENERIC_GET_VCO_COMMAND 32
#define HID_GENERIC_SAVE_GAIN_TABLE_COMMAND 33
#define HID_GENERIC_GET_GAIN_TABLE_COMMAND 34
#define HID_GENERIC_SET_BPF_COMMAND 35
#define HID_GENERIC_SYNC_WORD_COMMAND 36
#define HID_GENERIC_HW_CPLD_SN_COMMAND 37
#define HID_GENERIC_IAP_COMMAND 38
#define HID_GENERIC_WRITE_HWORD_TO_FLASH 39
#define HID_GENERIC_BROADCAST_COMMAND 40
#define HID_GENERIC_GET_BROADCAST_STATE_COMMAND 41
#define HID_GENERIC_BYPASS_LPF_COMMAND 42
#define HID_GENERIC_GET_BYPASS_LPF_COMMAND 43
#define HID_GENERIC_SET_MULTICHANNEL_COMMAND 48
#define HID_GENERIC_MULTICAST_COMMAND 49
#define HID_GENERIC_SET_AFEDRI_ID_COMMAND 50
#define HID_GENERIC_GET_AFEDRI_ID_COMMAND 51
#define HID_GENERIC_SAVE_AFEDRI_ID_COMMAND 52
#define HID_GENERIC_SOFT_RESET_COMMAND 53
#define HID_GENERIC_GET_DHCP_STATE_COMMAND 54
#define HID_GENERIC_SET_DHCP_STATE_COMMAND 55
#define HID_GENERIC_SET_PHASE_SHIFT_COMMAND 56
#define HID_GENERIC_HW_CPLD2_SN_COMMAND 57
#define HID_GENERIC_I2C_SEND_COMMAND 58
#define HID_GENERIC_I2C_READ_COMMAND 59
#define HID_GENERIC_I2C_START_COND_COMMAND 60
#define HID_GENERIC_I2C_STOP_COND_COMMAND 61
#define HID_GENERIC_MULT_WRITE_JTAG_COMMAND 62
#define HID_GENERIC_GET_MAXIMUM_SR_COMMAND 63
#define HID_GENERIC_GET_MAIN_CLOCK_FREQ 67
#define HID_GENERIC_SET_USB_MODE        68
#define TCP_HID_PACKET 0x7
#define HID_DEVICE_VID 0x483
#define HID_DEVICE_PID 0x5932
#define HID_DEVICE_ATEECS_VID 0x255D
#define HID_DEVICE_ATEECS_PID 0x0001
#define HID_DEVICE_SDR_NET_PID 0x0002
#define HID_DEVICE_ATEECS_PID_6 0x0006

extern	int GainIndex[];
extern double GainTable[];
extern int FE_Gain;
extern	hid_device *fe_handle;
extern volatile int hid_status;

int HID_Device_Init(char * msg);
int HID_Device_Reconnect( char * msg);
int HID_Read_Data( char *buf);
int HID_Set_Frequency(unsigned long frequency);
int HID_ChangeFE_Gain(int index);
int HID_ChangeRF_Gain(int channel, int dac_value);
int HID_Check_FW_Version(int value);
int HID_Init_FE(void);
int HID_Get_Frequency(int channel);
int HID_Device_Enumerate(void);
unsigned long HID_Get_Generic_Report(
                  unsigned char *generic_command, const unsigned char *buf);
unsigned long HID_Read_Frequency(const unsigned char *buf);
long HID_Get_Init_Status(void);
int HID_console_On(int value);
int HID_Send_console_char(int value);
int HID_Get_EEPROM_Data(unsigned char VirtAddress);
int HID_Get_Main_Clock_Frequency(void);
int HID_Send_Main_Clock_Frequency(unsigned long main_frequency);
int HID_Send_EEPROM_Data(unsigned char VirtAddress, 
                                             unsigned short eeprom_data);
int HID_Send_defaults_to_EEPROM(void);
double RF_Gain_Int_2_Double(int value);
int RF_Gain_Calc(int value);
int RF_Gain_IntDb_2_Int(int gain);
int HID_Get_Sample_Rate(void);
int HID_Send_Sample_Rate(unsigned long SampleRate);
int HID_Get_Diversity_Mode(void);
int HID_Send_Diversity_Mode(short mode);
int HID_Send_Diversity_Sample_Rate(unsigned long SampleRate);
int HID_Get_SDR_IP_Address(void);
int HID_Get_SDR_IP_Mask(void);
int HID_Get_Gateway_IP_Address(void);
int HID_Get_DST_IP_Address(void);
int HID_Get_UDP_Port(void);
int HID_Get_Serial_Number(void);
int HID_Send_Generic_Command_No_Param(unsigned char command);
int HID_Set_New_Sample_Rate(unsigned long SampleRate);
int HID_Save_SDR_IP_Address(const unsigned long ip_address);
int HID_Save_SDR_IP_Mask(const unsigned long ip_mask);
int HID_Save_Gateway_IP_Address(const unsigned long ip_address);
int HID_Save_DST_IP_Address(const unsigned long ip_address);
int HID_Save_UDP_Port(const unsigned long port);
int HID_Send_Generic_Long_Data(unsigned char command, 
                                               unsigned long eeprom_data);
int HID_Start_UDP_Stream(const unsigned long command);
int HID_Set_Broadcast_Flag(const unsigned long command);
int HID_Get_Broadcast_Flag(void);
int HID_Send_LPF_Bypass(short command);
void hid_change_tcp_network_flag(int state);
int hid_save_recieved_tcp_packet(char *data, int size);
int HID_Change_VCO_voltage(int vco_value);
int HID_Save_Default_VCO_voltage(int vco_value);
int HID_Get_VCO_voltage(void);
int HID_Save_Gain_Table(unsigned short gain_table);
int HID_Get_Gain_Table(void);
int HID_Change_BPF_Tune(unsigned short bpf_value);
int HID_Get_Sync_State(void);
int HID_Get_HW_CPLD_SN(void);
int HID_Firmware_Upgrade(void);
int HID_Send_To_Flash_HalfWord(unsigned short address, unsigned short data);
int HID_Set_Multichannel_Mode(int mode);
int HID_Set_Multicast_Flag(const unsigned long command);
int HID_SoftReset_SDR(void);
int HID_Set_ID_String(const unsigned long command);
int HID_Save_Default_ID(const unsigned long command);
int HID_Get_Emulation_ID(void);
int HID_Set_DHCP_Server_State(int command);
int HID_Get_DHCP_Server_State(void);
int HID_Set_Phase_Shift(int phase);
int HID_Get_CPLD2_SN(void);
int HID_Close(void);
int HID_Send_USB_Mode(short mode);
int SetMSDPath(const char *p_name);
void MSD_Envy_Init(void);
int Set_USB_Serial_number(wchar_t *serial);

#if LIBUDEV_PRESENT == 1

extern int  HID_COMMAND_SIZE;

char *afedriusb_parm_text[MAX_AFEDRIUSB_PARM]={"VID",
                                               "PID",
                                               "Serno1",
                                               "Serno2",
                                               "Serno3",
                                               "Serno4",
                                               "Freq adjust",
                                               "Check"};
P_AFEDRIUSB afedriusb;
char *afedriusb_parfil_name="par_afedriusb";



#define NON_BLOCKING_HID_IO
#define MAX_AFEDRI_TYPES 4

int vids[MAX_AFEDRI_TYPES]={HID_DEVICE_ATEECS_VID,
                            HID_DEVICE_ATEECS_VID,
                            HID_DEVICE_VID,
                            HID_DEVICE_ATEECS_VID};            

int pids[MAX_AFEDRI_TYPES]={HID_DEVICE_SDR_NET_PID,
                            HID_DEVICE_ATEECS_PID_6,
                            HID_DEVICE_PID,
                            HID_DEVICE_ATEECS_PID};
hid_device *fe_handle=NULL;

int cmd_sizes[MAX_AFEDRI_TYPES]={7,7,5,5};
#define MAX_UNITS 10


void afedriusb_rx_amp_control(void)
{
int rf_gain;
int fe_gain;
int i;
char report[HID_COMMAND_MAX_SIZE]={0};
fg.gain_increment = 1;
// Limit gain to -10 to +41dB.
if(fg.gain < -10)fg.gain = -10;
if(fg.gain > 41)fg.gain = 41;
if(fg.gain >= 35)
  {
  rf_gain=35;
  fe_gain=fg.gain-35;
  }
else
  {
  rf_gain=fg.gain;
  fe_gain=0;
  }  
report[0]=HID_GENERIC_REPORT;
report[1]=HID_GENERIC_DAC_COMMAND & 0xFF;
report[2]= ((rf_gain + 10) * 8) / 3;
report[3]= 0;
for(i=4; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
hid_write(fe_handle, (const unsigned char *)report, HID_COMMAND_SIZE);
report[0] = HID_GENERIC_REPORT;
report[1]=  HID_GENERIC_GAIN_COMMAND & 0xFF;
//int GainIndex[7]={0,4,2,6,1,5,3};
//double GainTable[7]={1.00, 1.14, 1.33, 1.60, 2.00, 2.67, 4};

report[2]= fe_gain+1;
for(i=3; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
hid_write(fe_handle, (const unsigned char *)report, HID_COMMAND_SIZE);
}

void afedriusb_rx_freq_control(void)
{
unsigned long frequency;
int i;
unsigned char report[HID_COMMAND_MAX_SIZE]={0};
if(fe_handle == NULL)return;
frequency=(fg.passband_center*(100000000-afedriusb.freq_adjust))/100;
report[0] = HID_FREQUENCY_REPORT;
for(i=0; i < 4;  i++)
  {
  report[i+1]= (frequency >> (i*8)) & 0xFF;
  }
for(i=5; i < HID_COMMAND_MAX_SIZE;  i++) report[i]=0;
hid_write(fe_handle, (const unsigned char *)report, HID_COMMAND_SIZE);
}


void close_afedriusb_control(void)
{
hid_close(fe_handle);
}

void open_afedriusb_control(void)
{
int i;
int *ise;
struct hid_device_info *devs, *cur_dev;
char s[80];
i=read_sdrpar(afedriusb_parfil_name, MAX_AFEDRIUSB_PARM, 
                             afedriusb_parm_text, (int*)((void*)&afedriusb));
if(i != 0 || afedriusb.check != AFEDRIUSBPAR_VERNR)
  {
  lirerr(1389);
  goto afu_init_err;
  }
ise=(int*)s;   
devs = hid_enumerate(afedriusb.vid, afedriusb.pid);
cur_dev = devs;
while (cur_dev) 
  {
  sprintf(s,"%ls               ",cur_dev->serial_number);
  if(afedriusb.sernum1 == ise[0] &&
     afedriusb.sernum2 == ise[1] &&
     afedriusb.sernum3 == ise[2] &&
     afedriusb.sernum4 == ise[3])goto match;
  cur_dev = cur_dev->next;
  }
hid_free_enumeration(devs);
  {
  lirerr(1390);
  goto afu_init_err;
  }
match:;
fe_handle=hid_open_path(cur_dev->path);
hid_free_enumeration(devs);
if(fe_handle == NULL)lirerr(1390);
afu_init_err:;
}

void afedriusb_setup(void)
{
FILE *afedriusb_file;
int type, line, unit;
char s[128];
int i, k;
int *ise, *sdr_pi;
// Enumerate and print the HID devices on the system
struct hid_device_info *devs, *cur_dev;
int afedri_vid[MAX_UNITS];
int afedri_pid[MAX_UNITS];
char afedri_sernum[MAX_UNITS][40];
line=2;
k=0;
clear_screen();
for(type=0; type<MAX_AFEDRI_TYPES; type++)
  {
  devs = hid_enumerate(vids[type], pids[type]);
  cur_dev = devs;
  while (cur_dev) 
    {
    sprintf(s,"%2d: Product %ls",k,cur_dev->product_string);
    lir_text(2,line,s);
    sprintf(s,"Type: %04hx %04hx",cur_dev->vendor_id, cur_dev->product_id);
    lir_text(36,line,s);
    afedri_vid[k]=cur_dev->vendor_id;
    afedri_pid[k]=cur_dev->product_id;
    sprintf(s,"Serial: %ls",cur_dev->serial_number);
    lir_text(53,line,s);
    sprintf(afedri_sernum[k],"%ls               ",cur_dev->serial_number);
    k++;
    line++;
    if(k >= MAX_UNITS)goto list_end;
    cur_dev = cur_dev->next;
    }
  hid_free_enumeration(devs);
  }
list_end:;  
line++;  
if(k == 0)
  {
  lir_text(5,line,"No Afedri device found on the USB system.");
  lir_text(5,line+2,press_any_key);
  await_processed_keyboard();
  return;
  }
if(k == 1)
  {
  lir_text(5,line,"Unit autoselected.");  
  lir_text(5,line+2,press_any_key);
  unit=0;
  }
else
  {
  lir_text(5,line,"Select radio hardware by line number. =>");
  unit=lir_get_integer(46,line,2,0,k-1);
  }
line+=2;
afedriusb.vid=afedri_vid[unit];
afedriusb.pid=afedri_pid[unit];
ise=(int*)afedri_sernum[unit];
afedriusb.sernum1=ise[0];
afedriusb.sernum2=ise[1];
afedriusb.sernum3=ise[2];
afedriusb.sernum4=ise[3];
afedriusb.check=AFEDRIUSBPAR_VERNR;
afedriusb_file=fopen(afedriusb_parfil_name,"w");
lir_text(5,line,"LO frequency error in ppb =>");
afedriusb.freq_adjust=lir_get_integer(34,line,7,-500000,500000);
if(afedriusb_file == NULL)
  {
  lirerr(384268);
  return;
  }
sdr_pi=(int*)(&afedriusb);
for(i=0; i<MAX_AFEDRIUSB_PARM; i++)
  {
  fprintf(afedriusb_file,"%s [%d]\n",afedriusb_parm_text[i],sdr_pi[i]);
  }
parfile_end(afedriusb_file);
}
#endif


void init_netafedri(void)
{
}
