// Copyright (c) <2012> <Leif Asbrink>
//
// Permission is hereby granted, free of charge, to any person 
// obtaining a copy of this software and associated documentation 
// files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of 
// the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be 
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
// OR OTHER DEALINGS IN THE SOFTWARE.


#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "globdef.h"
#include "uidef.h"
#include "sdrdef.h"
#include "conf.h"
#include "fft1def.h"
#include "screendef.h"
#include "rusage.h"
#include "thrdef.h"
#include "options.h"

#define MAX_LIBUSB_BULK_BUFFER 16320


#if PERSEUSLIB_PRESENT > 0
#include <perseus-sdr.h>
perseus_descr *descr;
#endif
#define MAX_SDR14_DRIVER_NAMES 4
char *sdr14_driver_names[MAX_SDR14_DRIVER_NAMES]=
                            { "/dev/linrad_ft245",
                              "/dev/ft245?",
                              "/dev/ttyUSB?",
                              "/dev/usb/ttyUSB?"};
// Perseus receiver USB buffering scheme definitions (never touch)
#define PERSEUS_SAMPLESPERFRAME	170   // Samples in one FX2 USB transaction
#define PERSEUS_COMPONENTSPERSAMPLE 2 // I & Q components
#define PERSEUS_BYTESPERCOMPONENT   3 // 24 bit data
// Derived defines
#define PERSEUS_BYTESPERSAMPLE (PERSEUS_BYTESPERCOMPONENT*PERSEUS_COMPONENTSPERSAMPLE)
#define PERSEUS_BYTESPERFRAME (PERSEUS_SAMPLESPERFRAME*PERSEUS_BYTESPERSAMPLE)

extern P_PERSEUS pers;
extern float perseus_filter_cutoff[MAX_PERSEUS_FILTER+1];
extern char *perseus_bitstream_names[MAX_PERSEUS_RATES];
int perseus_bytesperbuf;

void start_excalibur_read(void){}
void lir_excalibur_read(void){}
void set_excalibur_frequency(void){}
void excalibur_stop(void){}

#define MAX_SDR14_DEVICES 6
int sdr14_scan_usb(void);
typedef struct {
int sernum1;
int sernum2;
int sernum3;
int device_code;
}SDR14_DEVICEINFO;

SDR14_DEVICEINFO sdr14_device[MAX_SDR14_DEVICES]; 
int detected_device_no;


void sdr14_scan_result(int *line, char* dev, int verify)
{
char s[80], ss[80], sn[80];
int *isn;
int len;
int j, code;
char csn[12];
isn = (int*)(void*)&csn[0];
sdr_target_name(sn);
code=UNDEFINED_DEVICE_CODE;
if( strcmp(sdr14_name_string,sn) == 0)
  {
  code=SDR14_DEVICE_CODE;
  } 
if( strcmp(sdriq_name_string,sn) == 0)
  {
  code=SDRIQ_DEVICE_CODE;
  } 
if(code != UNDEFINED_DEVICE_CODE)
  {
  sdr_target_serial_number(ss);
  len=(int)strlen(ss)-1;
  j=11;
  while(len >= 0 && j >= 0)
    {
    csn[j]=ss[len];
    j--;
    len--;
    }
  while(j >= 0)
    {
    csn[j]=0;
    j--;
    }
  if(verify)
    {
    if(  sdr14.sernum1 == isn[0] &&
         sdr14.sernum2 == isn[1] &&
         sdr14.sernum3 == isn[2] &&
         sdr14.device_code == code)
      {
      return;
      }   
    }
  else
    {
    sdr14_device[line[0]].sernum1=isn[0];
    sdr14_device[line[0]].sernum2=isn[1];
    sdr14_device[line[0]].sernum3=isn[2];
    sdr14_device[line[0]].device_code=code;
    sprintf(s,"Line:%d:  Name:%s    Serial no:%s    Device %s",line[0],sn,ss,dev);
    SNDLOG"\n%s",s);
    lir_text(5,2+line[0],s);
    line[0]++;
    if(line[0] >= MAX_SDR14_DEVICES)
      {
      close_sdr14();
      lirerr(1125);
      return;
      }
    }
  }
close_sdr14();
}

int sdr14_scan_usb(void)
{
char s[80];
int i, j, selected_device, line;
size_t len;
selected_device=-1;
line=0;
for(i=0; i<MAX_SDR14_DRIVER_NAMES; i++)
  {
  len=strlen(sdr14_driver_names[i]);
  if(sdr14_driver_names[i][len-1] != '?')
    {
    SNDLOG"\nTrying to open %s",sdr14_driver_names[i]);
    sdr=open( sdr14_driver_names[i] ,O_RDWR );
    if(sdr != -1)
      {
      sdr14_scan_result(&line, sdr14_driver_names[i], FALSE);
      if(kill_all_flag)return -1;
      close_sdr14();
      }
    }
  else
    {
    for(j=0; j<8; j++)
      {
      strcpy(s,sdr14_driver_names[i]);
      s[len-1]='0'+j;
      SNDLOG"\nTrying to open %s",s);
      if(write_log)fflush(sndlog);
      sdr=open(s ,O_RDWR );
      if(sdr != -1)
        {
        sdr14_scan_result(&line, s, FALSE);
        if(kill_all_flag)return -1;
        close_sdr14();
        }
      }
    }  
  }  
clear_lines(10,12);
if(line > 0)
  {
  lir_text(0,line+4,"Select device by line number");
  selected_device=lir_get_integer(29,line+4,1,0,line-1);
  sdr14.sernum1=sdr14_device[selected_device].sernum1;
  sdr14.sernum2=sdr14_device[selected_device].sernum2;
  sdr14.sernum3=sdr14_device[selected_device].sernum3;
  sdr14.device_code=sdr14_device[selected_device].device_code;
  ui.rx_addev_no=sdr14_device[selected_device].device_code;
  }
return selected_device;
}

void open_sdr14(void)
{
char s[80];
int i, j, line;
size_t len;
line=0;
for(i=0; i<MAX_SDR14_DRIVER_NAMES; i++)
  {
  len=strlen(sdr14_driver_names[i]);
  if(sdr14_driver_names[i][len-1] != '?')
    {
    SNDLOG"\nTrying to open %s",sdr14_driver_names[i]);
    if(write_log)fflush(sndlog);
    sdr=open( sdr14_driver_names[i] ,O_RDWR );
    if(sdr != -1)
      {
      sdr14_scan_result(&line, sdr14_driver_names[i], TRUE);
      if(kill_all_flag)return;
      }
    if(sdr != -1)return;  
    }
  else
    {
    for(j=0; j<8; j++)
      {
      strcpy(s,sdr14_driver_names[i]);
      s[len-1]='0'+j;
      SNDLOG"\nTrying to open %s",s);
      if(write_log)fflush(sndlog);
      sdr=open(s ,O_RDWR );
      if(sdr != -1)
        {
        sdr14_scan_result(&line, s, TRUE);
        if(kill_all_flag)return;
        }
      if(sdr != -1)return;  
      }
    }  
  }  
}




int load_excalibur_library(char* buf)
{
(void)buf;
return 0;
}


void open_excalibur(void)
{
}

void close_excalibur(void)
{
}

void select_excalibur_mode(int *line)
{
(void)line;
}

void set_excalibur_att(void)
{
/*
int i;
i=-fg.gain;
//ig31_device->lpVtbl->SetAttenuator(ig31_device, i);
*/
}


void set_perseus_frequency(void)
{
#if PERSEUSLIB_PRESENT > 0
double dt1;
dt1=fg.passband_center*1000000;
if(dt1 < 0 || dt1 > adjusted_sdr_clock/2)
  {
  fg.passband_center=7.05;
  dt1=fg.passband_center*1000000;
  }
dt1/=adjusted_sdr_clock/PERSEUS_SAMPLING_CLOCK;  
perseus_set_ddc_center_freq(descr, dt1, pers.presel);
#endif
}

void fill_perseus_rates(void)
{
#if PERSEUSLIB_PRESENT == 1
if (perseus_get_sampling_rates (descr, perseus_rates, PERSEUS_RATES_BUFSIZE))
  {
  lirerr(1196);
  return;
  }
no_of_perseus_rates = 0;
while (perseus_rates[no_of_perseus_rates] > 1) 
  {
  no_of_perseus_rates++;
  }
#endif
#if PERSEUSLIB_PRESENT == 2
int pr[PERSEUS_RATES_BUFSIZE];
double dt1;
if (perseus_get_sampling_rates (descr, pr, PERSEUS_RATES_BUFSIZE))
  {
  lirerr(1196);
  return;
  }
no_of_perseus_rates = 0;
while (pr[no_of_perseus_rates] > 1)
  {
  switch (pr[no_of_perseus_rates])
    {
    case 48000:
    dt1=PERSEUS_SAMPLING_CLOCK/1664;
    break;
    
    case 95000:
    dt1=PERSEUS_SAMPLING_CLOCK/840;
    break;
    
    case 96000:
    dt1=PERSEUS_SAMPLING_CLOCK/832;
    break;
    
    case 192000:
    dt1=PERSEUS_SAMPLING_CLOCK/416;
    break;
    
    default:
    dt1=(double)pr[no_of_perseus_rates];
    }
  perseus_rates[no_of_perseus_rates]=dt1;
  no_of_perseus_rates++;
  }
#endif
}

void perseus_stop(void)
{
#if PERSEUSLIB_PRESENT > 0
perseus_stop_async_input(descr);
lir_sleep(10000);
#endif
}



int perseus_callback(void *buf, int buf_size, void *extra)
{
int i;
register char c1;
char *pBuf;
(void)extra;
pBuf=(char*)buf;
c1=0;
// This callback gets called whenever a buffer is ready
// from the USB interface
for(i=0; i<buf_size; i+=6)
  {
  timf1_char[timf1p_sdr+3]=pBuf[i+2];
  c1=pBuf[i+2]&0x80;
  timf1_char[timf1p_sdr+2]=pBuf[i+1];
  timf1_char[timf1p_sdr+1]=pBuf[i  ];
  c1|=pBuf[i];
  timf1_char[timf1p_sdr  ]=0;
  timf1_char[timf1p_sdr+7]=pBuf[i+5];
  timf1_char[timf1p_sdr+6]=pBuf[i+4];
  timf1_char[timf1p_sdr+5]=pBuf[i+3];
  timf1_char[timf1p_sdr+4]=0;
  timf1p_sdr=(timf1p_sdr+8)&timf1_bytemask;
  }
if( (c1&1) != 0)
  {
  ad_maxamp[0]=0x7fff;
  ad_maxamp[1]=0x7fff;
  }
if(((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) >=
              (int)snd[RXAD].block_bytes)lir_set_event(EVENT_PERSEUS_INPUT);
return 0;
}

void start_perseus_read(void)
{
#if PERSEUSLIB_PRESENT > 0
int i;
sdr=-1;
perseus_bytesperbuf=snd[RXAD].block_bytes/PERSEUS_BYTESPERFRAME;
perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
if(perseus_bytesperbuf > MAX_LIBUSB_BULK_BUFFER)
  {
  perseus_bytesperbuf=MAX_LIBUSB_BULK_BUFFER/PERSEUS_BYTESPERFRAME;
  perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
  }
timf1p_sdr=timf1p_pa;
lir_sched_yield();
i=perseus_start_async_input(descr,perseus_bytesperbuf, perseus_callback, NULL);
if(i < 0) 
  {
  lirerr(1195);
  return;
  }
sdr=0;
#endif
}

void lir_perseus_read(void)
{
while(((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) < (int)snd[RXAD].block_bytes)
  {
  lir_await_event(EVENT_PERSEUS_INPUT);
  }
}

int perseus_load_proprietary_code(int rate_no)
{
#if PERSEUSLIB_PRESENT > 0
int i;
i=perseus_set_sampling_rate_n(descr, rate_no);
if(i<0) return 1194;
return 0;
#else
return rate_no;
#endif
}

int perseus_eeprom_read(unsigned char *buf, int addr, unsigned char count)
{
#if PERSEUSLIB_PRESENT > 0
eeprom_prodid *prodid;
int i;
char s[80];
prodid=(eeprom_prodid*) buf;
(void) buf;
(void) addr;
(void) count;
i=perseus_get_product_id(descr,prodid);
if(i<0)
  {
  sprintf(s,"get product id error: %s", perseus_errorstr());
  clear_screen();
  lir_text(5,5,s);
  lir_text(5,7,press_any_key);
  await_processed_keyboard();
  lirerr(1193);
  return 1;
  }
return 0;
#else
buf[0]=addr;
return count;
#endif
}

void open_perseus(void)
{
#if PERSEUSLIB_PRESENT > 0
int i;
#if PERS_DEBUG == 1
perseus_set_debug(3);
#endif
if(sdr != -1)
  {
#if PERS_DEBUG == 1
  fprintf(STDERR,"lsdr.c: error sdr=-1\n");
#endif
  return;
  }
if(perseus_init() == 0)
  {
#if PERS_DEBUG == 1
  fprintf(STDERR,"lsdr.c: error perseus_init() failed\n");
#endif
  lir_text(3,3,"Could not init. Is Perseus connected with power on?");
conerr:;
  return;
  }
descr=perseus_open(0);
if(descr == NULL)
  {
#if PERS_DEBUG == 1
  fprintf(STDERR,"lsdr.c: error perseus_open() failed\n");
#endif
  clear_screen();
  lir_text(3,2,"Could not open Perseus. Do you have write permission to USB?");
  lir_text(3,3,"Maybe you should use sudo (or run as root)");
  goto conerr;
  }
i=perseus_firmware_download(descr,NULL);
if(i < 0)
  {
#if PERS_DEBUG == 1
  fprintf(STDERR,"lsdr.c: error perseus_firmware_download(...) failed\n");
#endif
  close_perseus();
  return;
  }
sdr=0;  
#endif
}

void perseus_store_att(unsigned char idAtt)
{
#if PERSEUSLIB_PRESENT > 0
perseus_set_attenuator(descr, idAtt);
#else
(void)idAtt;
#endif
}

void perseus_store_presel(unsigned char idPresel)
{
(void) idPresel;
}

void close_perseus(void)
{
#if PERSEUSLIB_PRESENT > 0
perseus_close(descr);
perseus_exit();
sdr=-1;
#endif
}


void lir_sdr14_write(char *s, int bytes)
{
int i, totwr;
totwr=0;
do_write:;
i=write(sdr,&s[totwr],bytes-totwr);
if(i < 0)return;
totwr+=i;
if(totwr == bytes)return;
lir_sched_yield();
goto do_write;
}

int lir_sdr14_read(char *s,int bytes)
{
int i, totread;
totread=0;
do_read:;
i=read(sdr, &s[totread], bytes-totread);
if(i < 0)return i;
totread+=i;
if(totread == bytes)return bytes;
lir_sched_yield();
goto do_read;
}

void close_sdr14(void)
{
int i;
i=fcntl(sdr, F_GETFD);
i|=O_NONBLOCK;
fcntl(sdr,F_SETFD,i);
lir_sleep(10000);
close(sdr);
sdr=-1;
}

